/*
 * Copyright (c) 2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.pat.sftptest.SFTPFunktionen
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.sftptest.SFTPFunktionen; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.ftp;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.*;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse stellt statischen SFTP-Funktionen zu Verfgung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 282 $ / $Date: 2012-08-30 13:42:53 +0200 (Do, 30 Aug 2012) $ / ($Author: LG $)
 */
public class SFTPFunktionen {
    private static final String SFTP = "sftp";

    //~ METHODEN ==============================================================

    /**
     * Ermittle alle Dateiennamen im angegebenen SFTP-Ordner.
     *
     * @param session Session.
     * @param sftpOrdner SFTP-Ordner.
     *
     * @return Liste der Dateiennamen im angegebenen SFTP-Ordner
     * oder null, falls nicht ermittelbar ist.
     */
    public static List<String> ermittleDateienImSFTPOrdner(Session session, String sftpOrdner) {
        List<String> listeDateien = new ArrayList<String>();
        ChannelSftp  sftpChannel  = null;

        try {

            // ---------------------------------------------------------
            // ffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            java.util.Vector vOrdnerInhalt;

            if (sftpOrdner.isEmpty()) {
                vOrdnerInhalt = sftpChannel.ls("/");
            } else {
                vOrdnerInhalt = sftpChannel.ls(sftpOrdner);
            }

            if (vOrdnerInhalt != null) {
                for (Object obj : vOrdnerInhalt) {
                    if (obj instanceof ChannelSftp.LsEntry) {
                        ChannelSftp.LsEntry lse = (ChannelSftp.LsEntry) obj;

                        if (!lse.getAttrs().isDir()) {
                            listeDateien.add(lse.getFilename());
                        }
                    }
                }
            }
        }
        catch (Exception e) {
            listeDateien = null;
        }

        schliesseSftpKanal(sftpChannel);

        return listeDateien;
    }

    /**
     * Schliesse den SFTP-Kanal.
     *
     * @param sftpChannel ChannelSftp
     */
    public static void schliesseSftpKanal(ChannelSftp sftpChannel) {

        // Versuche den Kanal zu schlieen:
        try {
            if (sftpChannel != null) {
                sftpChannel.disconnect();
            }
        }
        catch (Exception ignored) {}
    }

    /**
     * bertrage lokale Datei auf SFTP-Server.
     * <p/>
     * Lokale Datei wird zuerst in SFTP-Lock-Datei bertragen.
     * Falls die Gren der lokalen und bertragenen Dateien gleich sind,
     * wird die SFTP-Datei umbenannt.
     *
     * @param session SFTP-Session.
     * @param dateiLokal Die lokale zu bertragende Datei.
     * @param dateiSftp SFTP-Datei als Ergebnis der bertragung.
     * @param dateiSftpLock SFTP-Lock-Datei, wohin die lokale Datei kopiert werden soll.
     *
     * @return Exception, falls beim bertragen _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception uebertrageLokaleDateiAufSFTPServer(Session session, File dateiLokal, String dateiSftp, String dateiSftpLock) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // ffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // bertrage lokale Datei auf SFTPServer in SFTP-Lock-Datei:
            try {
                sftpChannel.put(dateiLokal.getAbsolutePath(), dateiSftpLock);
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("Datei <%s> konnte nicht auf SFTP-Server in <%s> bertragen werden (%s)", dateiLokal.getAbsolutePath(), dateiSftpLock, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }

            //
            // ---------------------------------------------------------
            // Falls die Gren der lokalen und bertragen Dateien gleich sind,
            // umbenenne die Datei auf dem SFTP-Server:
            long sizeLokal = dateiLokal.length();
            long sizeSftp  = sftpChannel.stat(dateiSftpLock).getSize();

            if (sizeSftp == sizeLokal) {
                try {

                    // Lsche Datei (falls existiert):
                    try {
                        sftpChannel.rm(dateiSftp);
                    }
                    catch (Exception ignored) {}

                    // Umbenenne Datei:
                    sftpChannel.rename(dateiSftpLock, dateiSftp);
                    schliesseSftpKanal(sftpChannel);

                    return null;
                }
                catch (Exception e) {
                    Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in <%s> umbennant werden", dateiSftpLock, dateiSftp), e);

                    e2.setStackTrace(e.getStackTrace());
                    schliesseSftpKanal(sftpChannel);

                    return e2;
                }
            } else {
                schliesseSftpKanal(sftpChannel);

                return new Exception(String.format("Die bertragene SFTP-Datei <%s> hat falsche Gre (%d statt %d)", dateiSftpLock, sizeSftp, sizeLokal));
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schlieen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }

    /**
     * bertrage die SFTP-Datei in lokale Datei.
     *
     * @param session Session.
     * @param dateiSftp Die zu bertragende SFTP-Datei.
     * @param dateiLokal Lokale Datei, wohin die SFTP-Datei kopiert werden soll.
     *
     * @return Exception, falls beim bertragen _Fehler aufgetreten sind,
     *  null - sonst.
     */
    public static Exception uebertrageSFTPDateiInLokaleDatei(Session session, String dateiSftp, String dateiLokal) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // ffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // bertrage die SFTP-Datei in lokaler Datei:
            try {
                sftpChannel.get(dateiSftp, dateiLokal);
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in lokale Datei <%s> bertragen werden (%s)", dateiSftp, dateiLokal, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }

            // ---------------------------------------------------------
            // Prfe die Gren:
            long sizeLokal = new File(dateiLokal).length();
            long sizeSftp  = sftpChannel.stat(dateiSftp).getSize();

            if (sizeSftp == sizeLokal) {
                schliesseSftpKanal(sftpChannel);

                return null;
            } else {
                schliesseSftpKanal(sftpChannel);

                return new Exception(String.format("Die bertragene lokale Datei <%s> hat falsche Gre (%d statt %d)", dateiLokal, sizeLokal, sizeSftp));
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schlieen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort
     *
     * @return SFTP-Session.
     *
     * @throws Exception Exception, falls bei der Verbindung _Fehler aufgetreten sind.
     */
    public static Session verbindeMitSFTPServer(String url, String benutzer, String passwort) throws Exception {
        return verbindeMitSFTPServer(url, 22, benutzer, passwort);
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param port SFTP-Port.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort
     *
     * @return SFTP-Session.
     */
    public static Session verbindeMitSFTPServer(String url, int port, String benutzer, String passwort) throws Exception {
        JSch jsch = new JSch();

        // -----------------------------------------
        // TODO Host-Datei definieren (ber Aufrufparameter)
        // -----------------------------------------
        // jsch.setKnownHosts("sftp_inovat\\key\\priv");
        //
        // -----------------------------------------
        // Session ffnen:
        // -----------------------------------------
        Session session = jsch.getSession(benutzer, url, port);

        session.setPassword(passwort);

        /*
        // -----------------------------------------
        // TODO Proxy-Einstellungen:
        // -----------------------------------------
        if (_sftpProxy) {
            // via HTTP proxy:
            session.setProxy(new ProxyHTTP(_sftpProxyHost, _sftpPort));
            // via SOCKS proxy
            session.setProxy(new ProxySOCKS5(_sftpProxyHost, _sftpPort));
        }
        */

        // -----------------------------------------
        // HostKey:
        // -----------------------------------------
        // TODO berprfung der HostKeys ausschalten <--- ber Aufrufparameter.
        // WICHTIG: es wird automatisch allen Hosts geglaubt, die HostKeys werden nicht berprft.
        // Durch den Eintrag <config.put("StrictHostKeyChecking", "no")> wird explizit eine Verbindung
        // zu jeden beliebigen Host erlaubt (die Strikte berprfung der HostKeys wird unterbinden).
        java.util.Properties config = new java.util.Properties();

        config.put("StrictHostKeyChecking", "no");
        session.setConfig(config);

        // -----------------------------------------
        // Verbindung:
        // -----------------------------------------
        session.connect();

        return session;
    }

    /**
     * Verschiebe bzw. umbenenne SFTP-Datei.
     *
     * @param session Session.
     * @param dateiSftpAlt Die alte SFTP-Datei (mit Path).
     * @param dateiSftpNeu Die neue SFTP-Datei (mit Path).
     *
     * @return Exception, falls beim Verschieben _Fehler aufgetreten sind,
     *  null - sonst.
     */
    public static Exception verschiebeSFTPDatei(Session session, String dateiSftpAlt, String dateiSftpNeu) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // ffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // Verschiebe bzw. umbenenne die SFTP-Datei:
            try {
                sftpChannel.rename(dateiSftpAlt, dateiSftpNeu);
                schliesseSftpKanal(sftpChannel);

                return null;
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in <%s> verschiebt/umbenannt werden (%s)", dateiSftpAlt, dateiSftpNeu, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schlieen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 13.06.13
