/*
 * Copyright 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.ftp;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.Session;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.ArrayList;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse dient zur Verwaltung der gesamten SFTP Kommunikation.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 0 $ / $Date: neu $ / ($Author: HCK $)
 */
public class SFTPKommunikation {

    /** TODO Fileseparator auf dem Server. */
    private final static String FILESEPARATOR_SERVER = "/";

    /** DebugLogger fr Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** Passwort zum Login auf den SFTP-Server. */
    private String _passwd;

    /** Port des SFTP-Servers. */
    private int _port;

    /** Typ des Systemobjekte fr den BASt-Band Import/Export. */
    private String _typBAStBand;

    /** Verzeichnis das berwacht wird. */
    private String _ueberwachungsVerezichnis;

    /** URL des SFTP-Servers. */
    private String _url;

    /** User fr den Login auf den SFTP-Server. */
    private String _user;

    /** Verzeichnis in das die Dateien nach erfolgreichem Import verschoben werden. */
    private String _verschiebeVerzeichnis;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor.
     *
     * @param dav                      ClientDavInterface.
     * @param url                      URL des SFTP-Servers.
     * @param port                     Port des SFTP-Servers.
     * @param user                     User des SFTP-Servers.
     * @param passwd                   Passwort fr den Login auf den SFTP-Server.
     * @param typBAStBand              Typ des Systemobjekte fr den BASt-Band Import/Export.
     * @param ueberwachungsVerzeichnis das zu berwachende Verzeichnis.
     * @param verschiebeVerzeichnis    das Verzeichnis, in das die Dateien nach erfolgreichem Import verschoben werden.
     */
    public SFTPKommunikation(ClientDavInterface dav, String url, int port, String user, String passwd, String typBAStBand, String ueberwachungsVerzeichnis, String verschiebeVerzeichnis) {
        _dav                      = dav;
        _url                      = url;
        _port                     = port;
        _user                     = user;
        _passwd                   = passwd;
        _typBAStBand              = typBAStBand;
        _ueberwachungsVerezichnis = ueberwachungsVerzeichnis;
        _verschiebeVerzeichnis    = verschiebeVerzeichnis;
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle SFTP-Datei-Pfad.
     *
     * @param sftpDateiName SFTP-Dateiname.
     * @param sftpOrdner SFTP-Pfad, wo die Datei liegt.
     *
     * @return SFTP-Datei-Pfad.
     */
    private String ermittleSftpDateiPfad(String sftpDateiName, String sftpOrdner) {
        if (sftpOrdner.isEmpty()) {
            return sftpDateiName;
        } else {
            if (sftpOrdner.endsWith(FILESEPARATOR_SERVER)) {
                return String.format("%s%s", sftpOrdner, sftpDateiName);
            } else {
                return String.format("%s%s%s", sftpOrdner, FILESEPARATOR_SERVER, sftpDateiName);
            }
        }
    }

    /**
     * Die Methode zur berwachung des Verzeichnisses.
     */
    public void ueberwacheVerzeichnis() {

        // ---------------------------------------------------------------
        // Erstelle die SFTP-Verbindung:
        // ---------------------------------------------------------------
        Session session;

        try {
            session = SFTPFunktionen.verbindeMitSFTPServer(_url, _port, _user, _passwd);
        }
        catch (Exception e) {
            session = null;
            debug.warning(String.format("Fehler bei der Verbindung mit dem SFTP-Server:\n%s", e.getLocalizedMessage()), e.getStackTrace());
        }

        if (session != null) {

            // ---------------------------------------------------------------
            // Ermittle alle Dateien im SFTP-berwachungs-Verzeichnis:
            // ---------------------------------------------------------------
            List<String> alleDateien = SFTPFunktionen.ermittleDateienImSFTPOrdner(session, _ueberwachungsVerezichnis);

            if (alleDateien == null) {
                debug.warning(String.format("Die Dateien knnen nicht aus dem berwachungsverzeichnis <%s> ermittelt werden", _ueberwachungsVerezichnis));
                alleDateien = new ArrayList<String>();
            }

            // ---------------------------------------------------------------
            // Bearbeite alle SFTP-Dateien:
            // ---------------------------------------------------------------
            for (String dateiSftp : alleDateien) {
                String dateiSftpMitPfad = ermittleSftpDateiPfad(dateiSftp, _ueberwachungsVerezichnis);

                // ---------------------------------------------------------------
                // Falls Bearbeitungs-Datei --->
                // verschiebe die in SFTP-Verzeichnis _verschiebeVerzeichnis:
                // ---------------------------------------------------------------
                if (dateiSftp.endsWith(Verzeichnisueberwachung.BEARBEITUNGS_DATEI)) {
                    String    neu = ermittleSftpDateiPfad(dateiSftp, _verschiebeVerzeichnis);
                    Exception e   = SFTPFunktionen.verschiebeSFTPDatei(session, dateiSftpMitPfad, neu);

                    if (e != null) {
                        debug.warning(String.format("Datei <%s> kann nicht in %s verschoben bzw. umbenannt werden:\n%s", dateiSftpMitPfad, neu, e.getLocalizedMessage()), e.getStackTrace());
                    }
                }

                // ---------------------------------------------------------------
                // Falls nicht Bearbeitungs-Datei und nicht Lock-Datei --->
                // Sende berwachungsdaten fr die Datei an DaV:
                // ---------------------------------------------------------------
                else if (!dateiSftp.endsWith(Verzeichnisueberwachung.LOCK_DATEI)) {
                    DavKommunikation davKomm = new DavKommunikation(_dav, _typBAStBand);

                    davKomm.sendeVerzeichnisUeberwachungsDaVDaten(dateiSftp, _ueberwachungsVerezichnis, false, _url, _port, _user, _passwd, false);
                }
            }

            // ---------------------------------------------------------------
            // Terminiere die SFTP-Verbindung:
            // ---------------------------------------------------------------
            session.disconnect();
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.06.13
